@@ -8,7 +8,7 @@ from collections import namedtuple
 
 __author__ = 'Measurement Standards Laboratory of New Zealand'
 __copyright__ = '\xa9 2017 - 2021, ' + __author__
-__version__ = '2.4.1'
+__version__ = '2.5.0'
 
 _v = re.search(r'(\d+)\.(\d+)\.(\d+)[.-]?(.*)', __version__).groups()
 
@@ -23,9 +23,10 @@ from .utils import (
     installed,
     info,
     set_log_level,
+    outdated_pypi_packages,
 )
 from .update import update
 from .create import create
 from .install import install
 from .uninstall import uninstall
-from .authorize import authorize
+from .authorise import authorise
