@@ -20,7 +20,6 @@ import logging
 import textwrap
 import platform
 import datetime
-import tempfile
 import threading
 import subprocess
 import collections
@@ -36,13 +35,20 @@ from colorama import Fore, Style, Back, init
 
 from . import _PKG_NAME
 
-_NUM_QUIET = 0
-_IS_WINDOWS = sys.platform in {'win32', 'cygwin'}
+_pip_quiet = 0
+_IS_WINDOWS = sys.platform == 'win32'
 
-if not os.path.isdir(os.path.join(os.path.expanduser('~'), '.msl')):
-    os.mkdir(os.path.join(os.path.expanduser('~'), '.msl'))
+_HOME_DIR = os.path.join(os.path.expanduser('~'), '.msl', 'package-manager')
+if not os.path.isdir(_HOME_DIR):
+    os.makedirs(_HOME_DIR)
 
-_GITHUB_AUTH_PATH = os.path.join(os.path.expanduser('~'), '.msl', '.mslpm-github-auth')
+_GITHUB_AUTH_PATH = os.path.join(_HOME_DIR, 'github-auth')
+
+# the HOME_DIR changed from ~/.msl to ~/.msl/package-manager
+# move a previously-created github-auth file to the new HOME_DIR
+_old_auth_path = os.path.join(os.path.expanduser('~'), '.msl', '.mslpm-github-auth')
+if os.path.isfile(_old_auth_path):
+    os.rename(_old_auth_path, _GITHUB_AUTH_PATH)
 
 _package_name_regex = re.compile(
     r'(?P<package_name>[*]?[\w-]*[*]?[\w-]*)(?P<extras_require>\[.*\])?(?P<version_requested>[<!=>~].*)?'
@@ -125,7 +131,7 @@ def github(update_cache=False):
             response = urlopen(Request(url, headers=headers))
         except HTTPError as err:
             if err.code == 401:
-                msg = 'You have provided invalid Authorization information'
+                msg = 'You have provided invalid authorisation information'
             elif err.code == 404:
                 # Could get this error if the URL does not exist.
                 # For example, getting .../releases/latest might not exist
@@ -138,8 +144,8 @@ def github(update_cache=False):
                     reset = int(reply['resources']['core']['reset'])
                     hms = datetime.datetime.fromtimestamp(reset).strftime('%H:%M:%S')
                     msg = 'The GitHub API rate limit was exceeded. Retry again at {} or create a file\n' \
-                          'with your GitHub authorization credentials to increase your rate limit. Run\n' \
-                          '"msl authorize --help" for more details.'.format(hms)
+                          'with your GitHub authorisation credentials to increase your rate limit. Run\n' \
+                          '"msl authorise --help" for more details.'.format(hms)
                 else:
                     msg = 'Unhandled HTTP error 403. The rate_limit was not reached...'
             else:
@@ -173,11 +179,11 @@ def github(update_cache=False):
             return cached_pgks
         return dict()
 
-    # check if the user specified their github authorization credentials
+    # check if the user specified their github authorisation credentials
     #
-    # the os.environ option is used for testing on travis and appveyor
-    # and it is not the recommended way for a user to store the credentials
-    auth = os.environ.get('GITHUB_AUTHORIZATION')
+    # the os.environ option is used for CI testing and it is not the
+    # recommended way for a user to store their credentials
+    auth = os.environ.get('MSL_PM_GITHUB_AUTH')
     if not auth and os.path.isfile(_GITHUB_AUTH_PATH):
         with open(_GITHUB_AUTH_PATH, 'rb') as fp:
             line = fp.readline().strip()
@@ -212,7 +218,7 @@ def github(update_cache=False):
         t.join()
 
     with open(path, 'w') as fp:
-        json.dump(pkgs, fp)
+        json.dump(pkgs, fp, indent=2)
 
     return _sort_packages(pkgs)
 
@@ -320,12 +326,14 @@ def installed():
             repo_name = dist.project_name
 
         description = ''
+        requires = []
         if dist.has_metadata(dist.PKG_INFO):
             for line in dist.get_metadata_lines(dist.PKG_INFO):
                 if line.startswith('Summary:'):
                     description = line[8:].strip()  # can be UNKNOWN
                 elif line.startswith('Home-page:') and 'github.com/MSLNZ' in line:
                     repo_name = line.split('/')[-1]
+                    requires = dist.requires()
                     if description:
                         break
 
@@ -338,12 +346,90 @@ def installed():
         pkgs[dist.project_name] = {
             'version': dist.version,
             'description': description,
-            'repo_name': repo_name
+            'repo_name': repo_name,
+            'requires': requires,
         }
 
     return _sort_packages(pkgs)
 
 
+def outdated_pypi_packages(msl_installed=None):
+    """Check PyPI for all non-MSL packages that are outdated.
+
+    .. versionadded:: 2.5.0
+
+    Parameters
+    ----------
+    msl_installed : :class:`dict`, optional
+        The MSL packages that are installed. If not specified
+        then calls :func:`installed` to determine the
+        installed packages.
+
+    Returns
+    -------
+    :class:`dict`
+        The information about the PyPI packages that are outdated.
+    """
+    pkgs_to_update = dict()
+    try:
+        output = subprocess.check_output([sys.executable, '-m', 'pip', 'list', '--outdated'])
+    except subprocess.CalledProcessError as e:
+        log.error('ERROR: "pip list --outdated" invalid -> {}'.format(e))
+        return pkgs_to_update
+
+    lines = output.decode().splitlines()
+    if not lines:
+        return pkgs_to_update
+
+    header = [item.lower() for item in lines[0].split()]
+    if header[:3] != ['package', 'version', 'latest']:
+        log.error('ERROR: pip has changed the structure of the outdated packages')
+        return pkgs_to_update
+
+    # ignore lines[1] since it is a bunch of '-'
+    outdated_packages = [dict(zip(header, line.split())) for line in lines[2:]]
+
+    if not msl_installed:
+        msl_installed = installed()
+
+    for msl_project_name, item in msl_installed.items():
+        for outdated in outdated_packages:
+            package = outdated['package']
+            if package == 'pip':
+                continue
+
+            for msl_requirement in item['requires']:
+                if msl_requirement.project_name.startswith(package):
+                    # cannot update a package to the latest version
+                    # on PyPI if the installed MSL package specifies
+                    # that it only supports a specific version
+                    if msl_requirement.specifier:
+                        specifier = str(msl_requirement.specifier)
+                        if package in pkgs_to_update:
+                            if specifier not in pkgs_to_update[package]['version']:
+                                # multiple version constraints
+                                pkgs_to_update[package]['version'] += ','+specifier
+                        else:
+                            pkgs_to_update[package] = {
+                                'installed_version': outdated['version'],
+                                'using_pypi': True,
+                                'extras_require': '',
+                                'version': specifier,
+                                'repo_name': '',
+                            }
+
+            if package not in pkgs_to_update and package not in msl_installed:
+                pkgs_to_update[package] = {
+                    'installed_version': outdated['version'],
+                    'using_pypi': True,
+                    'extras_require': '',
+                    'version': outdated['latest'],
+                    'repo_name': '',
+                }
+
+    return _sort_packages(pkgs_to_update)
+
+
 def pypi(update_cache=False):
     """Get the information about the MSL packages_ that are available on PyPI.
 
@@ -417,7 +503,7 @@ def pypi(update_cache=False):
         return _inspect_github_pypi('pypi', False)[0]
 
     with open(path, mode='wt') as fp:
-        json.dump(pkgs, fp)
+        json.dump(pkgs, fp, indent=2)
 
     return _sort_packages(pkgs)
 
@@ -430,16 +516,16 @@ def set_log_level(level):
     level : :class:`int`
         A value from one of the :py:ref:`levels`.
     """
-    global _NUM_QUIET
+    global _pip_quiet
 
-    if level <= logging.WARNING:
-        _NUM_QUIET = 0
+    if level <= logging.INFO:
+        _pip_quiet = 0
+    elif level == logging.WARNING:
+        _pip_quiet = 1
     elif level == logging.ERROR:
-        _NUM_QUIET = 1
-    elif level == logging.CRITICAL:
-        _NUM_QUIET = 2
+        _pip_quiet = 2
     else:
-        _NUM_QUIET = 3
+        _pip_quiet = 3
 
     log.setLevel(level)
 
@@ -509,7 +595,7 @@ def _check_wildcards_and_prefix(names, pkgs):
     return _packages
 
 
-def _create_install_list(names, branch, tag, update_cache):
+def _create_install_list(names, branch, commit, tag, update_cache):
     """Create a list of package names to ``install`` that are GitHub repositories_.
 
     Parameters
@@ -517,9 +603,11 @@ def _create_install_list(names, branch, tag, update_cache):
     names : :class:`tuple` of :class:`str`
         The name of a single GitHub repository or multiple repository names.
     branch : :class:`str` or :data:`None`
-        The name of a GitHub branch.
+        The name of a git branch.
+    commit : :class:`str` or :data:`None`
+        The hash value of a git commit.
     tag : :class:`str` or :data:`None`
-        The name of a GitHub tag.
+        The name of a git tag.
     update_cache : :class:`bool`
         Whether to force the GitHub cache to be updated when you call this function.
 
@@ -528,8 +616,8 @@ def _create_install_list(names, branch, tag, update_cache):
     :class:`dict`
         The MSL packages to ``install``.
     """
-    zip_name = _get_github_zip_name(branch, tag)
-    if zip_name is None:
+    github_suffix = _get_github_url_suffix(branch, commit, tag)
+    if github_suffix is None:
         return
 
     # keep the order of the log messages consistent: pypi -> github -> local
@@ -548,7 +636,7 @@ def _create_install_list(names, branch, tag, update_cache):
     pkgs = {}
     for name, value in packages.items():
         if name in pkgs_installed or name in repo_names:
-            log.warning('The {!r} package is already installed'.format(name))
+            log.warning('The {!r} package is already installed -- use the update command'.format(name))
         elif name not in pkgs_github:
             log.error('Cannot install {!r} -- the package does not exist'.format(name))
         elif branch is not None and branch not in pkgs_github[name]['branches']:
@@ -614,62 +702,66 @@ def _get_input(msg):
         return input(msg)
 
 
-def _get_github_zip_name(branch, tag):
-    """Returns the name of a zip file in the GitHub archive.
+def _get_github_url_suffix(branch=None, commit=None, tag=None):
+    """Returns the URL suffix.
 
     Parameters
     ----------
     branch : :class:`str` or :data:`None`
-        The name of a GitHub branch.
+        The name of a git branch.
+    commit : :class:`str` or :data:`None`
+        The hash value of a git commit.
     tag : :class:`str` or :data:`None`
-        The name of a GitHub tag.
+        The name of a git tag.
 
     Returns
     -------
     :class:`str` or :data:`None`
-        The name of the zip file or :data:`None` if both `branch`
-        and `tag` were specified.
+        The suffix or :data:`None` if more than 1 of
+        `branch`, `tag` or `commit` were enabled.
     """
-    if branch is not None and tag is not None:
-        log.error('Cannot specify both a branch ({}) and a tag ({}).'.format(branch, tag))
-        return None
-    elif branch is None and tag is None:
-        return 'master'
-    elif branch is not None:
+    count = [bool(branch), bool(commit), bool(tag)].count(True)
+    if count == 0:
+        return 'main'  # default branch
+
+    if count > 1:
+        log.error('Can only specify a branch, a commit or a tag (not multiple options simultaneously)')
+        return
+
+    if branch:
         return branch
-    elif tag is not None:
-        return tag
-    else:
-        assert False, 'This branch ({}) and tag ({}) combo has not been handled'.format(branch, tag)
+
+    if commit:
+        return commit
+
+    return tag
 
 
 def _inspect_github_pypi(where, update_cache):
-    """Inspects the temp directory for the cached json file.
+    """Inspects the HOME directory for the cached json file.
 
     Parameters
     ----------
     where : :class:`str`
-        Either 'github' or 'pypi'
+        Either 'github' or 'pypi'.
     update_cache : :class:`bool`
         Whether to update the cache.
 
     Returns
     -------
     :class:`dict`
-        The packages
+        The packages.
     :class:`str`
-        The path to the cached files.
+        The path to the cached file.
     """
     if where == 'github':
-        filename = 'msl-github-repo-cache.json'
         suffix = 'GitHub repositories'
     elif where == 'pypi':
-        filename = 'msl-pypi-pkgs-cache.json'
         suffix = 'PyPI packages'
     else:
-        assert False, 'Inspecting GitHub or PyPI has been configured. Got {}'.format(where)
+        assert False, '{!r} != github or pypi'.format(where)
 
-    path = os.path.join(tempfile.gettempdir(), filename)
+    path = os.path.join(_HOME_DIR, where+'.json')
 
     cached_pgks = None
     if os.path.isfile(path):
@@ -687,7 +779,7 @@ def _inspect_github_pypi(where, update_cache):
     return dict(), path
 
 
-def _log_install_uninstall_message(packages, action, branch, tag, pkgs_pypi=None):
+def _log_install_uninstall_message(packages, action, branch=None, commit=None, tag=None, pkgs_pypi=None):
     """Print the ``install`` or ``uninstall`` summary for what is going to happen.
 
     Parameters
@@ -697,13 +789,13 @@ def _log_install_uninstall_message(packages, action, branch, tag, pkgs_pypi=None
     action : :class:`str`
         The text to show in color and in upper case about what's happening.
     branch : :class:`str` or :data:`None`
-        The name of a GitHub branch to use for the ``install``.
-        *Only used when installing packages*.
+        The name of a git branch to use. Only used when installing packages.
+    commit : :class:`str` or :data:`None`
+        The hash value of a git commit to use. Only used when installing packages.
     tag : :class:`str` or :data:`None`
-        The name of a GitHub tag to use for the ``install``.
-        *Only used when installing packages*.
+        The name of a git tag to use. Only used when installing packages.
     pkgs_pypi : :class:`dict`
-        Packages that are on PyPI. Only used when installing.
+        Packages that are on PyPI. Only used when installing packages.
     """
     pkgs = _sort_packages(packages)
 
@@ -726,12 +818,14 @@ def _log_install_uninstall_message(packages, action, branch, tag, pkgs_pypi=None
         if values.get('version_requested'):
             version = values['version_requested'].replace('==', '')
         else:
-            version = '' if branch is not None or tag is not None else values['version']
+            version = '' if (branch or commit or tag) else values['version']
         msg += '\n  {}  {} '.format(name.ljust(w[0]), version.ljust(w[1]))
         if action == 'REMOVED':
             continue
         if branch is not None:
             msg += ' [branch:{}]'.format(branch)
+        elif commit is not None:
+            msg += ' [commit:{}]'.format(commit[:7])
         elif tag is not None:
             msg += ' [tag:{}]'.format(tag)
         elif pkg in pkgs_pypi:
@@ -775,11 +869,10 @@ class _ColourStreamHandler(logging.StreamHandler):
 
     def emit(self, record):
         _ColourStreamHandler.previous_message = record.getMessage()
+        stream = sys.stdout if record.levelno < logging.WARNING else sys.stderr
         try:
-            message = self.format(record)
-            self.stream.write(self.COLOURS[record.levelname] + message)
-            self.stream.write(getattr(self, 'terminator', '\n'))
-            self.flush()
+            stream.write(self.COLOURS[record.levelname] + self.format(record) + '\n')
+            stream.flush()
         except:
             self.handleError(record)
 
